@@ -148,6 +148,8 @@ abstract class CControllerHost extends CController {
 		CArrayHelper::sort($hosts, [['field' => $filter['sort'], 'order' => $filter['sortorder']]]);
 
 		$view_curl = (new CUrl())->setArgument('action', 'host.view');
+		$paging_arguments = array_filter(array_intersect_key($filter, self::FILTER_FIELDS_DEFAULT));
+		array_map([$view_curl, 'setArgument'], array_keys($paging_arguments), $paging_arguments);
 
 		// Split result array and create paging.
 		$paging = CPagerHelper::paginate($filter['page'], $hosts, $filter['sortorder'], $view_curl);
