@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -43,6 +43,10 @@ class CControllerSystemWarning extends CController {
 
 		$data['messages'] = CMessageHelper::getMessages();
 
+		$data['messages'] = array_map(function (array $message): string {
+			return $message['message'];
+		}, $data['messages']);
+
 		$this->setResponse(new CControllerResponseData($data));
 	}
 }
