@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -95,7 +95,7 @@ $str = _s('%1$s (year %2$s)', $dbTrigger['description'], zbx_date2str(_x('Y', DA
 $x = imagesx($im) / 2 - imagefontwidth(4) * mb_strlen($str) / 2;
 imageText($im, 10, 0, $x, 14, $darkred, $str);
 
-$now = time(null);
+$now = time();
 $count_now = [];
 $true = [];
 $false = [];
@@ -169,7 +169,7 @@ imageText($im, 8, 0, $shiftX + 9, $sizeY + $shiftYup + 60, $black, _('Problems')
 
 if ($debug_mode) {
 	$str = sprintf('%0.2f', microtime(true) - $start_time);
-	$str = _s('Generated in %s sec', $str);
+	$str = _s('Generated in %1$s sec', $str);
 	$str_size = imageTextSize(6, 0, $str);
 	imageText($im, 6, 0, imagesx($im) - $str_size['width'] - 5, imagesy($im) - 5, $gray, $str);
 }
