@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,9 +26,7 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = _('Configuration of discovery rules');
 $page['file'] = 'host_discovery.php';
-$page['scripts'] = ['class.cviewswitcher.js', 'multilineinput.js', 'multiselect.js', 'items.js', 'textareaflexible.js',
-	'class.tab-indicators.js'
-];
+$page['scripts'] = ['multilineinput.js', 'items.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -194,6 +192,7 @@ $fields = [
 								],
 	'preprocessing' =>			[T_ZBX_STR, O_OPT, P_NO_TRIM,	null,	null],
 	'overrides' =>				[T_ZBX_STR, O_OPT, P_NO_TRIM,	null,	null],
+	'context' =>				[T_ZBX_STR, O_MAND, P_SYS,		IN('"host", "template"'),	null],
 	// actions
 	'action' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
 									IN('"discoveryrule.massdelete","discoveryrule.massdisable",'.
@@ -234,10 +233,17 @@ $fields = [
 	'filter_status' =>			[T_ZBX_INT, O_OPT, null,	IN([-1, ITEM_STATUS_ACTIVE, ITEM_STATUS_DISABLED]),
 									null
 								],
+	'backurl' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
 	// sort and sortorder
 	'sort' =>					[T_ZBX_STR, O_OPT, P_SYS, IN('"delay","key_","name","status","type"'),	null],
 	'sortorder' =>				[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
 ];
+
+if (getRequest('interfaceid') == INTERFACE_TYPE_OPT) {
+	unset($fields['interfaceid']);
+	unset($_REQUEST['interfaceid']);
+}
+
 check_fields($fields);
 
 $_REQUEST['params'] = getRequest($paramsFieldName, '');
@@ -266,6 +272,21 @@ if (getRequest('itemid', false)) {
 	}
 	$_REQUEST['hostid'] = $item['hostid'];
 	$host = reset($item['hosts']);
+
+	foreach ($item['overrides'] as &$override) {
+		if (!array_key_exists('operations', $override)) {
+			continue;
+		}
+
+		foreach ($override['operations'] as &$operation) {
+			if (array_key_exists('optag', $operation)) {
+				CArrayHelper::sort($operation['optag'], ['tag', 'value']);
+				$operation['optag'] = array_values($operation['optag']);
+			}
+		}
+		unset($operation);
+	}
+	unset($override);
 }
 elseif ($hostid) {
 	$hosts = API::Host()->get([
@@ -280,55 +301,62 @@ elseif ($hostid) {
 	}
 }
 
+// Validate backurl.
+if (hasRequest('backurl') && !CHtmlUrlValidator::validateSameSite(getRequest('backurl'))) {
+	access_deny();
+}
+
+$prefix = (getRequest('context') === 'host') ? 'web.hosts.' : 'web.templates.';
+
 /**
  * Filter.
  */
-$sort_field = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'name'));
-$sort_order = getRequest('sortorder', CProfile::get('web.'.$page['file'].'.sortorder', ZBX_SORT_UP));
+$sort_field = getRequest('sort', CProfile::get($prefix.$page['file'].'.sort', 'name'));
+$sort_order = getRequest('sortorder', CProfile::get($prefix.$page['file'].'.sortorder', ZBX_SORT_UP));
 
-CProfile::update('web.'.$page['file'].'.sort', $sort_field, PROFILE_TYPE_STR);
-CProfile::update('web.'.$page['file'].'.sortorder', $sort_order, PROFILE_TYPE_STR);
+CProfile::update($prefix.$page['file'].'.sort', $sort_field, PROFILE_TYPE_STR);
+CProfile::update($prefix.$page['file'].'.sortorder', $sort_order, PROFILE_TYPE_STR);
 
 if (hasRequest('filter_set')) {
-	CProfile::updateArray('web.host_discovery.filter.groupids', getRequest('filter_groupids', []), PROFILE_TYPE_ID);
-	CProfile::updateArray('web.host_discovery.filter.hostids', getRequest('filter_hostids', []), PROFILE_TYPE_ID);
-	CProfile::update('web.host_discovery.filter.name', getRequest('filter_name', ''), PROFILE_TYPE_STR);
-	CProfile::update('web.host_discovery.filter.key', getRequest('filter_key', ''), PROFILE_TYPE_STR);
-	CProfile::update('web.host_discovery.filter.type', getRequest('filter_type', -1), PROFILE_TYPE_INT);
-	CProfile::update('web.host_discovery.filter.delay', getRequest('filter_delay', ''), PROFILE_TYPE_STR);
-	CProfile::update('web.host_discovery.filter.lifetime', getRequest('filter_lifetime', ''), PROFILE_TYPE_STR);
-	CProfile::update('web.host_discovery.filter.snmp_oid', getRequest('filter_snmp_oid', ''), PROFILE_TYPE_STR);
-	CProfile::update('web.host_discovery.filter.state', getRequest('filter_state', -1), PROFILE_TYPE_INT);
-	CProfile::update('web.host_discovery.filter.status', getRequest('filter_status', -1), PROFILE_TYPE_INT);
+	CProfile::updateArray($prefix.'host_discovery.filter.groupids', getRequest('filter_groupids', []), PROFILE_TYPE_ID);
+	CProfile::updateArray($prefix.'host_discovery.filter.hostids', getRequest('filter_hostids', []), PROFILE_TYPE_ID);
+	CProfile::update($prefix.'host_discovery.filter.name', getRequest('filter_name', ''), PROFILE_TYPE_STR);
+	CProfile::update($prefix.'host_discovery.filter.key', getRequest('filter_key', ''), PROFILE_TYPE_STR);
+	CProfile::update($prefix.'host_discovery.filter.type', getRequest('filter_type', -1), PROFILE_TYPE_INT);
+	CProfile::update($prefix.'host_discovery.filter.delay', getRequest('filter_delay', ''), PROFILE_TYPE_STR);
+	CProfile::update($prefix.'host_discovery.filter.lifetime', getRequest('filter_lifetime', ''), PROFILE_TYPE_STR);
+	CProfile::update($prefix.'host_discovery.filter.snmp_oid', getRequest('filter_snmp_oid', ''), PROFILE_TYPE_STR);
+	CProfile::update($prefix.'host_discovery.filter.state', getRequest('filter_state', -1), PROFILE_TYPE_INT);
+	CProfile::update($prefix.'host_discovery.filter.status', getRequest('filter_status', -1), PROFILE_TYPE_INT);
 }
 elseif (hasRequest('filter_rst')) {
-	CProfile::deleteIdx('web.host_discovery.filter.groupids');
+	CProfile::deleteIdx($prefix.'host_discovery.filter.groupids');
 
-	if (count(CProfile::getArray('web.host_discovery.filter.hostids', [])) != 1) {
-		CProfile::deleteIdx('web.host_discovery.filter.hostids');
+	if (count(CProfile::getArray($prefix.'host_discovery.filter.hostids', [])) != 1) {
+		CProfile::deleteIdx($prefix.'host_discovery.filter.hostids');
 	}
 
-	CProfile::delete('web.host_discovery.filter.name');
-	CProfile::delete('web.host_discovery.filter.key');
-	CProfile::delete('web.host_discovery.filter.type');
-	CProfile::delete('web.host_discovery.filter.delay');
-	CProfile::delete('web.host_discovery.filter.lifetime');
-	CProfile::delete('web.host_discovery.filter.snmp_oid');
-	CProfile::delete('web.host_discovery.filter.state');
-	CProfile::delete('web.host_discovery.filter.status');
+	CProfile::delete($prefix.'host_discovery.filter.name');
+	CProfile::delete($prefix.'host_discovery.filter.key');
+	CProfile::delete($prefix.'host_discovery.filter.type');
+	CProfile::delete($prefix.'host_discovery.filter.delay');
+	CProfile::delete($prefix.'host_discovery.filter.lifetime');
+	CProfile::delete($prefix.'host_discovery.filter.snmp_oid');
+	CProfile::delete($prefix.'host_discovery.filter.state');
+	CProfile::delete($prefix.'host_discovery.filter.status');
 }
 
 $filter = [
-	'groups' => CProfile::getArray('web.host_discovery.filter.groupids', []),
-	'hosts' => CProfile::getArray('web.host_discovery.filter.hostids', []),
-	'name' => CProfile::get('web.host_discovery.filter.name', ''),
-	'key' => CProfile::get('web.host_discovery.filter.key', ''),
-	'type' => CProfile::get('web.host_discovery.filter.type', -1),
-	'delay' => CProfile::get('web.host_discovery.filter.delay', ''),
-	'lifetime' => CProfile::get('web.host_discovery.filter.lifetime', ''),
-	'snmp_oid' => CProfile::get('web.host_discovery.filter.snmp_oid', ''),
-	'state' => CProfile::get('web.host_discovery.filter.state', -1),
-	'status' => CProfile::get('web.host_discovery.filter.status', -1)
+	'groups' => CProfile::getArray($prefix.'host_discovery.filter.groupids', []),
+	'hosts' => CProfile::getArray($prefix.'host_discovery.filter.hostids', []),
+	'name' => CProfile::get($prefix.'host_discovery.filter.name', ''),
+	'key' => CProfile::get($prefix.'host_discovery.filter.key', ''),
+	'type' => CProfile::get($prefix.'host_discovery.filter.type', -1),
+	'delay' => CProfile::get($prefix.'host_discovery.filter.delay', ''),
+	'lifetime' => CProfile::get($prefix.'host_discovery.filter.lifetime', ''),
+	'snmp_oid' => CProfile::get($prefix.'host_discovery.filter.snmp_oid', ''),
+	'state' => CProfile::get($prefix.'host_discovery.filter.state', -1),
+	'status' => CProfile::get($prefix.'host_discovery.filter.status', -1)
 ];
 
 $filter_groupids = [];
@@ -348,13 +376,22 @@ if ($filter['groups']) {
 
 // Get hosts.
 if ($filter['hosts']) {
-	$filter['hosts'] = CArrayHelper::renameObjectsKeys(API::Host()->get([
-		'output' => ['hostid', 'name'],
-		'hostids' => $filter['hosts'],
-		'templated_hosts' => true,
-		'editable' => true,
-		'preservekeys' => true
-	]), ['hostid' => 'id']);
+	if (getRequest('context') === 'host') {
+		$filter['hosts'] = CArrayHelper::renameObjectsKeys(API::Host()->get([
+			'output' => ['hostid', 'name'],
+			'hostids' => $filter['hosts'],
+			'editable' => true,
+			'preservekeys' => true
+		]), ['hostid' => 'id']);
+	}
+	else {
+		$filter['hosts'] = CArrayHelper::renameObjectsKeys(API::Template()->get([
+			'output' => ['templateid', 'name'],
+			'templateids' => $filter['hosts'],
+			'editable' => true,
+			'preservekeys' => true
+		]), ['templateid' => 'id']);
+	}
 
 	$filter_hostids = array_keys($filter['hosts']);
 
@@ -422,13 +459,13 @@ elseif (hasRequest('add') || hasRequest('update')) {
 
 				if ($simple_interval_parser->parse($interval['delay']) != CParser::PARSE_SUCCESS) {
 					$result = false;
-					info(_s('Invalid interval "%1$s".', $interval['delay']));
+					error(_s('Invalid interval "%1$s".', $interval['delay']));
 					break;
 				}
 
 				if ($time_period_parser->parse($interval['period']) != CParser::PARSE_SUCCESS) {
 					$result = false;
-					info(_s('Invalid interval "%1$s".', $interval['period']));
+					error(_s('Invalid interval "%1$s".', $interval['period']));
 					break;
 				}
 
@@ -441,7 +478,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 
 				if ($scheduling_interval_parser->parse($interval['schedule']) != CParser::PARSE_SUCCESS) {
 					$result = false;
-					info(_s('Invalid interval "%1$s".', $interval['schedule']));
+					error(_s('Invalid interval "%1$s".', $interval['schedule']));
 					break;
 				}
 
@@ -478,7 +515,6 @@ elseif (hasRequest('add') || hasRequest('update')) {
 					$step['params'] = implode("\n", $step['params']);
 					break;
 
-				// ZBX-16642
 				case ZBX_PREPROC_CSV_TO_JSON:
 					if (!array_key_exists(2, $step['params'])) {
 						$step['params'][2] = ZBX_PREPROC_CSV_NO_HEADER;
@@ -494,12 +530,14 @@ elseif (hasRequest('add') || hasRequest('update')) {
 				'error_handler' => ZBX_PREPROC_FAIL_DEFAULT,
 				'error_handler_params' => ''
 			];
+
+			unset($step['sortorder']);
 		}
 		unset($step);
 
 		$newItem = [
 			'itemid' => getRequest('itemid'),
-			'interfaceid' => getRequest('interfaceid'),
+			'interfaceid' => getRequest('interfaceid', 0),
 			'name' => getRequest('name'),
 			'description' => getRequest('description'),
 			'key_' => getRequest('key'),
@@ -687,15 +725,30 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	}
 
 	if (hasRequest('add')) {
-		show_messages($result, _('Discovery rule created'), _('Cannot add discovery rule'));
+		if ($result) {
+			CMessageHelper::setSuccessTitle(_('Discovery rule created'));
+		}
+		else {
+			CMessageHelper::setErrorTitle(_('Cannot add discovery rule'));
+		}
 	}
 	else {
-		show_messages($result, _('Discovery rule updated'), _('Cannot update discovery rule'));
+		if ($result) {
+			CMessageHelper::setSuccessTitle(_('Discovery rule updated'));
+		}
+		else {
+			CMessageHelper::setErrorTitle(_('Cannot update discovery rule'));
+		}
 	}
 
 	if ($result) {
 		unset($_REQUEST['itemid'], $_REQUEST['form']);
 		uncheckTableRows($checkbox_hash);
+
+		if (hasRequest('backurl')) {
+			$response = new CControllerResponseRedirect(getRequest('backurl'));
+			$response->redirect();
+		}
 	}
 }
 elseif (hasRequest('action') && str_in_array(getRequest('action'), ['discoveryrule.massenable', 'discoveryrule.massdisable']) && hasRequest('g_hostdruleid')) {
@@ -788,9 +841,7 @@ if (hasRequest('form')) {
 		}
 	}
 
-	$data = getItemFormData($form_item, [
-		'is_discovery_rule' => true
-	]);
+	$data = getItemFormData($form_item, ['form' => getRequest('form'), 'is_discovery_rule' => true]);
 	$data['lifetime'] = getRequest('lifetime', DB::getDefault('items', 'lifetime'));
 	$data['evaltype'] = getRequest('evaltype');
 	$data['formula'] = getRequest('formula');
@@ -799,11 +850,14 @@ if (hasRequest('form')) {
 	$data['overrides'] = getRequest('overrides', []);
 	$data['host'] = $host;
 	$data['preprocessing_test_type'] = CControllerPopupItemTestEdit::ZBX_TEST_TYPE_LLD;
-	$data['preprocessing_types'] = CDiscoveryRule::$supported_preprocessing_types;
+	$data['preprocessing_types'] = CDiscoveryRule::SUPPORTED_PREPROCESSING_TYPES;
 	$data['display_interfaces'] = ($host['status'] == HOST_STATUS_MONITORED
-			|| $host['status'] == HOST_STATUS_NOT_MONITORED);
+		|| $host['status'] == HOST_STATUS_NOT_MONITORED
+	);
+	$data['backurl'] = getRequest('backurl');
 
 	if (!hasRequest('form_refresh')) {
+		$i = 0;
 		foreach ($data['preprocessing'] as &$step) {
 			if ($step['type'] == ZBX_PREPROC_SCRIPT) {
 				$step['params'] = [$step['params'], ''];
@@ -811,10 +865,13 @@ if (hasRequest('form')) {
 			else {
 				$step['params'] = explode("\n", $step['params']);
 			}
+			$step['sortorder'] = $i++;
 		}
 		unset($step);
 	}
 
+	CArrayHelper::sort($data['preprocessing'], ['sortorder']);
+
 	// update form
 	if (hasRequest('itemid') && !getRequest('form_refresh')) {
 		$data['lifetime'] = $item['lifetime'];
@@ -824,9 +881,7 @@ if (hasRequest('form')) {
 		$data['lld_macro_paths'] = $item['lld_macro_paths'];
 		$data['overrides'] = $item['overrides'];
 		// Sort overrides to be listed in step order.
-		CArrayHelper::sort($data['overrides'], [
-			['field' => 'step', 'order' => ZBX_SORT_UP]
-		]);
+		CArrayHelper::sort($data['overrides'], ['step']);
 	}
 	// clone form
 	elseif (hasRequest('clone')) {
@@ -834,15 +889,17 @@ if (hasRequest('form')) {
 		$data['form'] = 'clone';
 	}
 
-	// Sort interfaces to be listed starting with one selected as 'main'.
-	CArrayHelper::sort($data['interfaces'], [
-		['field' => 'main', 'order' => ZBX_SORT_DOWN]
-	]);
-
 	if ($data['type'] != ITEM_TYPE_JMX) {
 		$data['jmx_endpoint'] = ZBX_DEFAULT_JMX_ENDPOINT;
 	}
 
+	$data['counter'] = null;
+	if (hasRequest('conditions')) {
+		$conditions = getRequest('conditions');
+		krsort($conditions);
+		$data['counter'] = key($conditions) + 1;
+	}
+
 	// render view
 	if (!$has_errors) {
 		echo (new CView('configuration.host.discovery.edit', $data))->getOutput();
@@ -854,10 +911,11 @@ else {
 		'hostid' => (count($filter_hostids) == 1) ? reset($filter_hostids) : 0,
 		'sort' => $sort_field,
 		'sortorder' => $sort_order,
-		'profileIdx' => 'web.host_discovery.filter',
-		'active_tab' => CProfile::get('web.host_discovery.filter.active', 1),
+		'profileIdx' => $prefix.'host_discovery.filter',
+		'active_tab' => CProfile::get($prefix.'host_discovery.filter.active', 1),
 		'checkbox_hash' => $checkbox_hash,
-		'is_template' => true
+		'is_template' => true,
+		'context' => getRequest('context')
 	];
 
 	// Select LLD rules.
@@ -869,6 +927,7 @@ else {
 		'selectTriggers' => API_OUTPUT_COUNT,
 		'selectHostPrototypes' => API_OUTPUT_COUNT,
 		'editable' => true,
+		'templated' => ($data['context'] === 'template'),
 		'filter' => [],
 		'search' => [],
 		'sortfield' => $sort_field,
@@ -936,7 +995,6 @@ else {
 	}
 
 	$data['discoveries'] = API::DiscoveryRule()->get($options);
-	$data['discoveries'] = CMacrosResolverHelper::resolveItemNames($data['discoveries']);
 
 	switch ($sort_field) {
 		case 'delay':
@@ -981,7 +1039,7 @@ else {
 	CPagerHelper::savePage($page['file'], $page_num);
 
 	$data['paging'] = CPagerHelper::paginate($page_num, $data['discoveries'], $sort_order,
-		new CUrl('host_discovery.php')
+		(new CUrl('host_discovery.php'))->setArgument('context', $data['context'])
 	);
 
 	$data['parent_templates'] = getItemParentTemplates($data['discoveries'], ZBX_FLAG_DISCOVERY_RULE);
