@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -34,6 +34,10 @@ class CCol extends CTag {
 		return $this;
 	}
 
+	public function getColSpan() {
+		return $this->getAttribute('colspan') ?? 1;
+	}
+
 	public function setColSpan($value) {
 		$this->setAttribute('colspan', $value);
 
