@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,10 +19,13 @@
 **/
 
 
+/**
+ * @deprecated - use CFormGrid instead.
+ */
 class CFormList extends CList {
 
 	protected $editable = true;
-	protected $formInputs = ['ctextbox', 'cnumericbox', 'ctextarea', 'ccombobox', 'ccheckbox', 'cpassbox'];
+	protected $formInputs = ['ctextbox', 'cnumericbox', 'ctextarea', 'ccheckbox', 'cpassbox'];
 
 	public function __construct($id = null) {
 		parent::__construct();
@@ -73,7 +76,7 @@ class CFormList extends CList {
 	public function addInfo($text) {
 		$this->addItem(
 			[
-				(new CDiv(_('Info')))->addClass(ZBX_STYLE_TABLE_FORMS_TD_LEFT),
+				(new CDiv())->addClass(ZBX_STYLE_TABLE_FORMS_TD_LEFT),
 				(new CDiv(
 					(new CDiv($text))->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
 				))->addClass(ZBX_STYLE_TABLE_FORMS_TD_RIGHT)
