@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 class CSvgText extends CSvgTag {
 
 	public function __construct($x, $y, $text) {
-		parent::__construct('text', true);
+		parent::__construct('text');
 
 		$this->setAttribute('x', $x);
 		$this->setAttribute('y', $y);
