@@ -58,6 +58,8 @@ class CRouter {
 		'action.operation.get'						=> ['CControllerActionOperationGet',					'layout.json',			null],
 		'action.operation.validate'					=> ['CControllerActionOperationValidate',				'layout.json',			null],
 		'auditlog.list'								=> ['CControllerAuditLogList',							'layout.htmlpage',		'reports.auditlog.list'],
+		'audit.settings.edit'						=> ['CControllerAuditSettingsEdit',						'layout.htmlpage',		'administration.audit.settings.edit'],
+		'audit.settings.update'						=> ['CControllerAuditSettingsUpdate',					null,					null],
 		'authentication.edit'						=> ['CControllerAuthenticationEdit',					'layout.htmlpage',		'administration.authentication.edit'],
 		'authentication.update'						=> ['CControllerAuthenticationUpdate',					null,					null],
 		'autoreg.edit'								=> ['CControllerAutoregEdit',							'layout.htmlpage',		'administration.autoreg.edit'],
@@ -79,7 +81,6 @@ class CRouter {
 		'dashboard.print'							=> ['CControllerDashboardPrint',						'layout.htmlpage',		'monitoring.dashboard.print'],
 		'dashboard.properties.check'				=> ['CControllerDashboardPropertiesCheck',				'layout.json',			null],
 		'dashboard.properties.edit'					=> ['CControllerDashboardPropertiesEdit',				'layout.json',			'dashboard.properties.edit'],
-		'dashboard.share.edit'						=> ['CControllerDashboardShareEdit',					'layout.json',			'dashboard.share.edit'],
 		'dashboard.share.update'					=> ['CControllerDashboardShareUpdate',					'layout.json',			null],
 		'dashboard.update'							=> ['CControllerDashboardUpdate',						'layout.json',			null],
 		'dashboard.view'							=> ['CControllerDashboardView',							'layout.htmlpage',		'monitoring.dashboard.view'],
@@ -103,11 +104,18 @@ class CRouter {
 		'export.valuemaps'							=> ['CControllerExport',								'layout.export',		null],
 		'favourite.create'							=> ['CControllerFavouriteCreate',						'layout.javascript',	null],
 		'favourite.delete'							=> ['CControllerFavouriteDelete',						'layout.javascript',	null],
+		'geomaps.edit'								=> ['CControllerGeomapsEdit',							'layout.htmlpage',		'administration.geomaps.edit'],
+		'geomaps.update'							=> ['CControllerGeomapsUpdate',							null,					null],
 		'gui.edit'									=> ['CControllerGuiEdit',								'layout.htmlpage',		'administration.gui.edit'],
 		'gui.update'								=> ['CControllerGuiUpdate',								null,					null],
 		'hintbox.actionlist'						=> ['CControllerHintboxActionlist',						'layout.json',			'hintbox.actionlist'],
 		'hintbox.eventlist'							=> ['CControllerHintboxEventlist',						'layout.json',			'hintbox.eventlist'],
 		'host.dashboard.view'						=> ['CControllerHostDashboardView',						'layout.htmlpage',		'monitoring.host.dashboard.view'],
+		'host.list'									=> ['CControllerHostList',								'layout.htmlpage',		'configuration.host.list'],
+		'host.massdelete'							=> ['CControllerHostMassDelete',						'layout.json',			null],
+		'host.create'								=> ['CControllerHostCreate',							'layout.json',			null],
+		'host.edit'									=> ['CControllerHostEdit',								'layout.htmlpage',		'configuration.host.edit'],
+		'host.update'								=> ['CControllerHostUpdate',							'layout.json',			null],
 		'host.view'									=> ['CControllerHostView',								'layout.htmlpage',		'monitoring.host.view'],
 		'host.view.refresh'							=> ['CControllerHostViewRefresh',						'layout.json',			'monitoring.host.view.refresh'],
 		'hostmacros.list'							=> ['CControllerHostMacrosList',						'layout.json',			'hostmacros.list'],
@@ -153,8 +161,10 @@ class CRouter {
 		'popup.condition.actions'					=> ['CControllerPopupConditionActions',					'layout.json',			'popup.condition.common'],
 		'popup.condition.event.corr'				=> ['CControllerPopupConditionEventCorr',				'layout.json',			'popup.condition.common'],
 		'popup.condition.operations'				=> ['CControllerPopupConditionOperations',				'layout.json',			'popup.condition.common'],
+		'popup.dashboard.share.edit'				=> ['CControllerPopupDashboardShareEdit',				'layout.json',			'popup.dashboard.share.edit'],
 		'popup.discovery.check'						=> ['CControllerPopupDiscoveryCheck',					'layout.json',			'popup.discovery.check'],
 		'popup.generic'								=> ['CControllerPopupGeneric',							'layout.json',			'popup.generic'],
+		'popup.host.edit'							=> ['CControllerHostEdit',								'layout.json',			'popup.host.edit'],
 		'popup.httpstep'							=> ['CControllerPopupHttpStep',							'layout.json',			'popup.httpstep'],
 		'popup.import'								=> ['CControllerPopupImport',							'layout.json',			'popup.import'],
 		'popup.import.compare'						=> ['CControllerPopupImportCompare',					'layout.json',			'popup.import.compare'],
@@ -174,21 +184,27 @@ class CRouter {
 		'popup.scheduledreport.subscription.edit'	=> ['CControllerPopupScheduledReportSubscriptionEdit',	'layout.json',			'popup.scheduledreport.subscription'],
 		'popup.scheduledreport.test'				=> ['CControllerPopupScheduledReportTest',				'layout.json',			'popup.scheduledreport.test'],
 		'popup.scriptexec'							=> ['CControllerPopupScriptExec',						'layout.json',			'popup.scriptexec'],
+		'popup.service.edit'						=> ['CControllerPopupServiceEdit',						'layout.json',			'popup.service.edit'],
+		'popup.service.statusrule.edit'				=> ['CControllerPopupServiceStatusRuleEdit',			'layout.json',			'popup.service.statusrule.edit'],
+		'popup.service.time.edit'					=> ['CControllerPopupServiceTimeEdit',					'layout.json',			'popup.service.time.edit'],
 		'popup.services'							=> ['CControllerPopupServices',							'layout.json',			'popup.services'],
+		'popup.sla.edit'							=> ['CControllerPopupSlaEdit',							'layout.json',			'popup.sla.edit'],
+		'popup.sla.excludeddowntime.edit'			=> ['CControllerPopupSlaExcludedDowntimeEdit',			'layout.json',			'popup.sla.excludeddowntime.edit'],
 		'popup.tabfilter.delete'					=> ['CControllerPopupTabFilterDelete',					'layout.json',			null],
 		'popup.tabfilter.edit'						=> ['CControllerPopupTabFilterEdit',					'layout.json',			'popup.tabfilter.edit'],
 		'popup.tabfilter.update'					=> ['CControllerPopupTabFilterUpdate',					'layout.json',			null],
 		'popup.testtriggerexpr'						=> ['CControllerPopupTestTriggerExpr',					'layout.json',			'popup.testtriggerexpr'],
 		'popup.triggerexpr'							=> ['CControllerPopupTriggerExpr',						'layout.json',			'popup.triggerexpr'],
-		'popup.triggerwizard'						=> ['CControllerPopupTriggerWizard',					'layout.json',			'popup.triggerwizard'],
 		'popup.massupdate.host'						=> ['CControllerPopupMassupdateHost',					'layout.json',			'popup.massupdate.host'],
 		'popup.massupdate.item'						=> ['CControllerPopupMassupdateItem',					'layout.json',			'popup.massupdate.item'],
 		'popup.massupdate.itemprototype'			=> ['CControllerPopupMassupdateItem',					'layout.json',			'popup.massupdate.item'],
+		'popup.massupdate.service'					=> ['CControllerPopupMassupdateService',				'layout.json',			'popup.massupdate.service'],
 		'popup.massupdate.template'					=> ['CControllerPopupMassupdateTemplate',				'layout.json',			'popup.massupdate.template'],
 		'popup.massupdate.trigger'					=> ['CControllerPopupMassupdateTrigger',				'layout.json',			'popup.massupdate.trigger'],
 		'popup.massupdate.triggerprototype'			=> ['CControllerPopupMassupdateTrigger',				'layout.json',			'popup.massupdate.trigger'],
 		'popup.valuemap.edit'						=> ['CControllerPopupValueMapEdit',						'layout.json',			'popup.valuemap.edit'],
 		'popup.valuemap.update'						=> ['CControllerPopupValueMapUpdate',					'layout.json',			null],
+		'popup.tophosts.column.edit'				=> ['CControllerPopupTopHostsColumnEdit',				'layout.json',			'popup.tophosts.column.edit'],
 		'problem.view'								=> ['CControllerProblemView',							'layout.htmlpage',		'monitoring.problem.view'],
 		'problem.view.refresh'						=> ['CControllerProblemViewRefresh',					'layout.json',			null],
 		'problem.view.csv'							=> ['CControllerProblemView',							'layout.csv',			'monitoring.problem.view'],
@@ -209,7 +225,6 @@ class CRouter {
 		'regex.list'								=> ['CControllerRegExList',								'layout.htmlpage',		'administration.regex.list'],
 		'regex.test'								=> ['CControllerRegExTest',								null,					null],
 		'regex.update'								=> ['CControllerRegExUpdate',							null,					null],
-		'report.services'							=> ['CControllerReportServices',						'layout.htmlpage',		'report.services'],
 		'report.status'								=> ['CControllerReportStatus',							'layout.htmlpage',		'report.status'],
 		'scheduledreport.create'					=> ['CControllerScheduledReportCreate',					null,					null],
 		'scheduledreport.delete'					=> ['CControllerScheduledReportDelete',					null,					null],
@@ -224,6 +239,22 @@ class CRouter {
 		'script.list'								=> ['CControllerScriptList',							'layout.htmlpage',		'administration.script.list'],
 		'script.update'								=> ['CControllerScriptUpdate',							null,					null],
 		'search'									=> ['CControllerSearch',								'layout.htmlpage',		'search'],
+		'service.create'							=> ['CControllerServiceCreate',							'layout.json',			null],
+		'service.delete'							=> ['CControllerServiceDelete',							'layout.json',			null],
+		'service.list'								=> ['CControllerServiceList',							'layout.htmlpage',		'service.list'],
+		'service.list.refresh'						=> ['CControllerServiceListRefresh',					'layout.json',			'service.list.refresh'],
+		'service.list.edit'							=> ['CControllerServiceListEdit',						'layout.htmlpage',		'service.list.edit'],
+		'service.list.edit.refresh'					=> ['CControllerServiceListEditRefresh',				'layout.json',			'service.list.edit.refresh'],
+		'service.statusrule.validate'				=> ['CControllerServiceStatusRuleValidate',				'layout.json',			null],
+		'service.update'							=> ['CControllerServiceUpdate',							'layout.json',			null],
+		'sla.create'								=> ['CControllerSlaCreate',								'layout.json',			null],
+		'sla.delete'								=> ['CControllerSlaDelete',								'layout.json',			null],
+		'sla.disable'								=> ['CControllerSlaDisable',							'layout.json',			null],
+		'sla.excludeddowntime.validate'				=> ['CControllerSlaExcludedDowntimeValidate',			'layout.json',			null],
+		'sla.list'									=> ['CControllerSlaList',								'layout.htmlpage',		'sla.list'],
+		'sla.enable'								=> ['CControllerSlaEnable',								'layout.json',			null],
+		'sla.update'								=> ['CControllerSlaUpdate',								'layout.json',			null],
+		'slareport.list'							=> ['CControllerSlaReportList',							'layout.htmlpage',		'slareport.list'],
 		'system.warning'							=> ['CControllerSystemWarning',							'layout.warning',		'system.warning'],
 		'tabfilter.profile.update'					=> ['CControllerTabFilterProfileUpdate',				'layout.json',			null],
 		'template.dashboard.delete'					=> ['CControllerTemplateDashboardDelete',				null,					null],
@@ -272,8 +303,10 @@ class CRouter {
 		'widget.discovery.view'						=> ['CControllerWidgetDiscoveryView',					'layout.widget',		'monitoring.widget.discovery.view'],
 		'widget.favgraphs.view'						=> ['CControllerWidgetFavGraphsView',					'layout.widget',		'monitoring.widget.favgraphs.view'],
 		'widget.favmaps.view'						=> ['CControllerWidgetFavMapsView',						'layout.widget',		'monitoring.widget.favmaps.view'],
+		'widget.geomap.view'						=> ['CControllerWidgetGeoMapView',						'layout.widget',		'monitoring.widget.geomap.view'],
 		'widget.graph.view'							=> ['CControllerWidgetGraphView',						'layout.widget',		'monitoring.widget.graph.view'],
 		'widget.graphprototype.view'				=> ['CControllerWidgetIteratorGraphPrototypeView',		'layout.json',			null],
+		'widget.item.view'							=> ['CControllerWidgetItemView',						'layout.widget',		'monitoring.widget.item.view'],
 		'widget.hostavail.view'						=> ['CControllerWidgetHostAvailView',					'layout.widget',		'monitoring.widget.hostavail.view'],
 		'widget.map.view'							=> ['CControllerWidgetMapView',							'layout.widget',		'monitoring.widget.map.view'],
 		'widget.navtree.item.edit'					=> ['CControllerWidgetNavTreeItemEdit',					'layout.json',			'monitoring.widget.navtreeitem.edit'],
@@ -283,31 +316,22 @@ class CRouter {
 		'widget.problemhosts.view'					=> ['CControllerWidgetProblemHostsView',				'layout.widget',		'monitoring.widget.problemhosts.view'],
 		'widget.problems.view'						=> ['CControllerWidgetProblemsView',					'layout.widget',		'monitoring.widget.problems.view'],
 		'widget.problemsbysv.view'					=> ['CControllerWidgetProblemsBySvView',				'layout.widget',		'monitoring.widget.problemsbysv.view'],
+		'widget.slareport.view'						=> ['CControllerWidgetSlaReportView',					'layout.widget',		'monitoring.widget.slareport.view'],
 		'widget.svggraph.view'						=> ['CControllerWidgetSvgGraphView',					'layout.widget',		'monitoring.widget.svggraph.view'],
 		'widget.systeminfo.view'					=> ['CControllerWidgetSystemInfoView',					'layout.widget',		'monitoring.widget.systeminfo.view'],
+		'widget.tophosts.view'						=> ['CControllerWidgetTopHostsView',					'layout.widget',		'monitoring.widget.tophosts.view'],
 		'widget.trigover.view'						=> ['CControllerWidgetTrigOverView',					'layout.widget',		'monitoring.widget.trigover.view'],
 		'widget.url.view'							=> ['CControllerWidgetUrlView',							'layout.widget',		'monitoring.widget.url.view'],
 		'widget.web.view'							=> ['CControllerWidgetWebView',							'layout.widget',		'monitoring.widget.web.view'],
 
 		// legacy actions
 		'actionconf.php'				=> ['CLegacyAction', null, null],
-		'adm.housekeeper.php'			=> ['CLegacyAction', null, null],
-		'adm.iconmapping.php'			=> ['CLegacyAction', null, null],
-		'adm.images.php'				=> ['CLegacyAction', null, null],
-		'adm.macros.php'				=> ['CLegacyAction', null, null],
-		'adm.other.php'					=> ['CLegacyAction', null, null],
-		'adm.regexps.php'				=> ['CLegacyAction', null, null],
-		'adm.triggerdisplayoptions.php'	=> ['CLegacyAction', null, null],
-		'adm.triggerseverities.php'		=> ['CLegacyAction', null, null],
-		'adm.valuemapping.php'			=> ['CLegacyAction', null, null],
-		'adm.workingtime.php'			=> ['CLegacyAction', null, null],
 		'auditacts.php'					=> ['CLegacyAction', null, null],
 		'browserwarning.php'			=> ['CLegacyAction', null, null],
 		'chart.php'						=> ['CLegacyAction', null, null],
 		'chart2.php'					=> ['CLegacyAction', null, null],
 		'chart3.php'					=> ['CLegacyAction', null, null],
 		'chart4.php'					=> ['CLegacyAction', null, null],
-		'chart5.php'					=> ['CLegacyAction', null, null],
 		'chart6.php'					=> ['CLegacyAction', null, null],
 		'chart7.php'					=> ['CLegacyAction', null, null],
 		'disc_prototypes.php'			=> ['CLegacyAction', null, null],
@@ -318,7 +342,6 @@ class CRouter {
 		'hostgroups.php'				=> ['CLegacyAction', null, null],
 		'hostinventories.php'			=> ['CLegacyAction', null, null],
 		'hostinventoriesoverview.php'	=> ['CLegacyAction', null, null],
-		'hosts.php'						=> ['CLegacyAction', null, null],
 		'httpconf.php'					=> ['CLegacyAction', null, null],
 		'httpdetails.php'				=> ['CLegacyAction', null, null],
 		'image.php'						=> ['CLegacyAction', null, null],
@@ -328,22 +351,17 @@ class CRouter {
 		'index_sso.php'					=> ['CLegacyAction', null, null],
 		'items.php'						=> ['CLegacyAction', null, null],
 		'jsrpc.php'						=> ['CLegacyAction', null, null],
-		'latest.php'					=> ['CLegacyAction', null, null],
 		'maintenance.php'				=> ['CLegacyAction', null, null],
 		'map.php'						=> ['CLegacyAction', null, null],
-		'overview.php'					=> ['CLegacyAction', null, null],
 		'report2.php'					=> ['CLegacyAction', null, null],
 		'report4.php'					=> ['CLegacyAction', null, null],
-		'services.php'					=> ['CLegacyAction', null, null],
-		'srv_status.php'				=> ['CLegacyAction', null, null],
 		'sysmap.php'					=> ['CLegacyAction', null, null],
 		'sysmaps.php'					=> ['CLegacyAction', null, null],
 		'templates.php' 				=> ['CLegacyAction', null, null],
 		'toptriggers.php'				=> ['CLegacyAction', null, null],
 		'tr_events.php'					=> ['CLegacyAction', null, null],
 		'trigger_prototypes.php'		=> ['CLegacyAction', null, null],
-		'triggers.php'					=> ['CLegacyAction', null, null],
-		'usergrps.php'					=> ['CLegacyAction', null, null]
+		'triggers.php'					=> ['CLegacyAction', null, null]
 	];
 
 	/**
