@@ -72,6 +72,12 @@ class CMacroParser extends CParser {
 		if (array_key_exists('ref_type', $options)) {
 			$this->options['ref_type'] = $options['ref_type'];
 		}
+
+		if (is_array($this->options['macros'])) {
+			usort($this->options['macros'], static function (string $a, string $b) {
+				return strlen($b) <=> strlen($a);
+			});
+		}
 	}
 
 	/**
