@@ -38,7 +38,6 @@ function prepareSubfilterOutput($label, $data, $subfilter, $subfilterName) {
 				' ',
 				new CSup($element['count'])
 			]))
-				->addClass(ZBX_STYLE_NOWRAP)
 				->addClass(ZBX_STYLE_SUBFILTER)
 				->addClass(ZBX_STYLE_SUBFILTER_ENABLED);
 		}
@@ -67,9 +66,7 @@ function prepareSubfilterOutput($label, $data, $subfilter, $subfilterName) {
 					$link,
 					' ',
 					new CSup(($subfilter ? '+' : '').$element['count'])
-				]))
-					->addClass(ZBX_STYLE_NOWRAP)
-					->addClass(ZBX_STYLE_SUBFILTER);
+				]))->addClass(ZBX_STYLE_SUBFILTER);
 			}
 		}
 	}
@@ -111,7 +108,6 @@ function prepareTagsSubfilterOutput(array $data, array &$subfilter): array {
 				' ',
 				new CSup($tag['count'])
 			]))
-				->addClass(ZBX_STYLE_NOWRAP)
 				->addClass(ZBX_STYLE_SUBFILTER)
 				->addClass(ZBX_STYLE_SUBFILTER_ENABLED);
 		}
@@ -137,9 +133,7 @@ function prepareTagsSubfilterOutput(array $data, array &$subfilter): array {
 					$link,
 					' ',
 					new CSup(($subfilter ? '+' : '').$tag['count'])
-				]))
-					->addClass(ZBX_STYLE_NOWRAP)
-					->addClass(ZBX_STYLE_SUBFILTER);
+				]))->addClass(ZBX_STYLE_SUBFILTER);
 			}
 		}
 
@@ -149,7 +143,7 @@ function prepareTagsSubfilterOutput(array $data, array &$subfilter): array {
 	return $output;
 }
 
-function makeItemSubfilter(array &$filter_data, array $items = [], string $context) {
+function makeItemSubfilter(array &$filter_data, array $items, string $context) {
 	// subfilters
 	$table_subfilter = (new CTableInfo())
 		->addRow([
@@ -191,7 +185,10 @@ function makeItemSubfilter(array &$filter_data, array $items = [], string $conte
 				if ($name === 'subfilter_tags') {
 					continue;
 				}
-				$show_item &= $value;
+				if (!$value) {
+					$show_item = false;
+					break;
+				}
 			}
 
 			if ($show_item) {
@@ -211,7 +208,10 @@ function makeItemSubfilter(array &$filter_data, array $items = [], string $conte
 				if ($name == 'subfilter_hosts') {
 					continue;
 				}
-				$show_item &= $value;
+				if (!$value) {
+					$show_item = false;
+					break;
+				}
 			}
 			if ($show_item) {
 				$host = reset($item['hosts']);
@@ -229,7 +229,10 @@ function makeItemSubfilter(array &$filter_data, array $items = [], string $conte
 				if ($name == 'subfilter_types') {
 					continue;
 				}
-				$show_item &= $value;
+				if (!$value) {
+					$show_item = false;
+					break;
+				}
 			}
 			if ($show_item) {
 				$item_params['types'][$item['type']]['count']++;
@@ -250,7 +253,10 @@ function makeItemSubfilter(array &$filter_data, array $items = [], string $conte
 				if ($name == 'subfilter_value_types') {
 					continue;
 				}
-				$show_item &= $value;
+				if (!$value) {
+					$show_item = false;
+					break;
+				}
 			}
 			if ($show_item) {
 				$item_params['value_types'][$item['value_type']]['count']++;
@@ -270,7 +276,10 @@ function makeItemSubfilter(array &$filter_data, array $items = [], string $conte
 				if ($name == 'subfilter_status') {
 					continue;
 				}
-				$show_item &= $value;
+				if (!$value) {
+					$show_item = false;
+					break;
+				}
 			}
 			if ($show_item) {
 				$item_params['status'][$item['status']]['count']++;
@@ -290,7 +299,10 @@ function makeItemSubfilter(array &$filter_data, array $items = [], string $conte
 				if ($name == 'subfilter_state') {
 					continue;
 				}
-				$show_item &= $value;
+				if (!$value) {
+					$show_item = false;
+					break;
+				}
 			}
 			if ($show_item) {
 				$item_params['state'][$item['state']]['count']++;
@@ -310,7 +322,10 @@ function makeItemSubfilter(array &$filter_data, array $items = [], string $conte
 				if ($name == 'subfilter_inherited') {
 					continue;
 				}
-				$show_item &= $value;
+				if (!$value) {
+					$show_item = false;
+					break;
+				}
 			}
 			if ($show_item) {
 				if ($item['templateid'] == 0) {
@@ -335,7 +350,10 @@ function makeItemSubfilter(array &$filter_data, array $items = [], string $conte
 				if ($name == 'subfilter_with_triggers') {
 					continue;
 				}
-				$show_item &= $value;
+				if (!$value) {
+					$show_item = false;
+					break;
+				}
 			}
 			if ($show_item) {
 				if (count($item['triggers']) == 0) {
@@ -360,7 +378,10 @@ function makeItemSubfilter(array &$filter_data, array $items = [], string $conte
 				if ($name == 'subfilter_discovered') {
 					continue;
 				}
-				$show_item &= $value;
+				if (!$value) {
+					$show_item = false;
+					break;
+				}
 			}
 			if ($show_item) {
 				if ($item['flags'] == ZBX_FLAG_DISCOVERY_NORMAL) {
@@ -397,7 +418,10 @@ function makeItemSubfilter(array &$filter_data, array $items = [], string $conte
 				if ($name === 'subfilter_trends') {
 					continue;
 				}
-				$show_item &= $value;
+				if (!$value) {
+					$show_item = false;
+					break;
+				}
 			}
 
 			if ($show_item) {
@@ -429,7 +453,10 @@ function makeItemSubfilter(array &$filter_data, array $items = [], string $conte
 				if ($name === 'subfilter_history') {
 					continue;
 				}
-				$show_item &= $value;
+				if (!$value) {
+					$show_item = false;
+					break;
+				}
 			}
 
 			if ($show_item) {
@@ -473,7 +500,10 @@ function makeItemSubfilter(array &$filter_data, array $items = [], string $conte
 				if ($name === 'subfilter_interval') {
 					continue;
 				}
-				$show_item &= $value;
+				if (!$value) {
+					$show_item = false;
+					break;
+				}
 			}
 
 			if ($show_item) {
@@ -484,9 +514,7 @@ function makeItemSubfilter(array &$filter_data, array $items = [], string $conte
 
 	// output
 	if (count($item_params['tags']) > 1) {
-		$tags_output = prepareTagsSubfilterOutput($item_params['tags'], $filter_data['subfilter_tags'],
-			'subfilter_tags'
-		);
+		$tags_output = prepareTagsSubfilterOutput($item_params['tags'], $filter_data['subfilter_tags']);
 		$table_subfilter->addRow([$tags_output]);
 	}
 
@@ -594,25 +622,36 @@ function prepareItemHttpAgentFormData(array $item) {
 
 		foreach ($item['query_fields']['name'] as $index => $key) {
 			$value = $item['query_fields']['value'][$index];
+			$sortorder = $item['query_fields']['sortorder'][$index];
 
 			if ($key !== '' || $value !== '') {
-				$query_fields[] = [$key => $value];
+				$query_fields[$sortorder] = [$key => $value];
 			}
 		}
+
+		ksort($query_fields);
 		$item['query_fields'] = $query_fields;
 	}
 
 	if ($item['headers']) {
-		$headers = [];
+		$tmp_headers = [];
 
 		foreach ($item['headers']['name'] as $index => $key) {
 			$value = $item['headers']['value'][$index];
+			$sortorder = $item['headers']['sortorder'][$index];
 
 			if ($key !== '' || $value !== '') {
-				$headers[$key] = $value;
+				$tmp_headers[$sortorder] = [$key => $value];
 			}
 		}
 
+		ksort($tmp_headers);
+		$headers = [];
+
+		foreach ($tmp_headers as $key_value_pair) {
+			$headers[key($key_value_pair)] = reset($key_value_pair);
+		}
+
 		$item['headers'] = $headers;
 	}
 
@@ -783,10 +822,12 @@ function getItemFormData(array $item = [], array $options = []) {
 					&& array_key_exists('value', $data[$property])) {
 				foreach ($data[$property]['name'] as $index => $key) {
 					if (array_key_exists($index, $data[$property]['value'])) {
-						$values[] = [$key => $data[$property]['value'][$index]];
+						$sortorder = $data[$property]['sortorder'][$index];
+						$values[$sortorder] = [$key => $data[$property]['value'][$index]];
 					}
 				}
 			}
+			ksort($values);
 			$data[$property] = $values;
 		}
 
@@ -819,10 +860,8 @@ function getItemFormData(array $item = [], array $options = []) {
 
 	// Dependent item initialization by master_itemid.
 	if (array_key_exists('master_item', $item)) {
-		$expanded = CMacrosResolverHelper::resolveItemNames([$item['master_item']]);
-		$master_item = reset($expanded);
-		$data['master_itemid'] = $master_item['itemid'];
-		$data['master_itemname'] = $master_item['name_expanded'];
+		$data['master_itemid'] = $item['master_item']['itemid'];
+		$data['master_itemname'] = $item['master_item']['name'];
 		// Do not initialize item data if only master_item array was passed.
 		unset($item['master_item']);
 	}
@@ -880,7 +919,7 @@ function getItemFormData(array $item = [], array $options = []) {
 		}
 
 		$data['templates'] = makeItemTemplatesHtml($item['itemid'], getItemParentTemplates([$item], $flag), $flag,
-			CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES), $data['context']
+			CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES)
 		);
 	}
 
@@ -1334,11 +1373,37 @@ function getItemPreprocessing(CForm $form, array $preprocessing, $readonly, arra
 				break;
 
 			case ZBX_PREPROC_PROMETHEUS_PATTERN:
+				$step_param_2_value = (array_key_exists('params', $step) && array_key_exists(2, $step['params']))
+					? $step['params'][2]
+					: '';
+
+				if ($step_param_1_value === ZBX_PREPROC_PROMETHEUS_FUNCTION) {
+					$step_param_1_value = $step_param_2_value;
+					$step_param_2_value = '';
+				}
+
 				$params = [
 					$step_param_0->setAttribute('placeholder',
 						_('<metric name>{<label name>="<label value>", ...} == <value>')
 					),
-					$step_param_1->setAttribute('placeholder', _('<label name>'))
+					(new CSelect('preprocessing['.$i.'][params][1]'))
+						->addOptions(CSelect::createOptionsFromArray([
+							ZBX_PREPROC_PROMETHEUS_VALUE => _('value'),
+							ZBX_PREPROC_PROMETHEUS_LABEL => _('label'),
+							ZBX_PREPROC_PROMETHEUS_SUM => 'sum',
+							ZBX_PREPROC_PROMETHEUS_MIN => 'min',
+							ZBX_PREPROC_PROMETHEUS_MAX => 'max',
+							ZBX_PREPROC_PROMETHEUS_AVG => 'avg',
+							ZBX_PREPROC_PROMETHEUS_COUNT => 'count'
+						]))
+						->addClass('js-preproc-param-prometheus-pattern-function')
+						->setValue($step_param_1_value)
+						->setReadonly($readonly),
+					(new CTextBox('preprocessing['.$i.'][params][2]', $step_param_2_value))
+						->setTitle($step_param_2_value)
+						->setAttribute('placeholder', _('<label name>'))
+						->setEnabled($step_param_1_value === ZBX_PREPROC_PROMETHEUS_LABEL)
+						->setReadonly($readonly)
 				];
 				break;
 
@@ -1348,7 +1413,6 @@ function getItemPreprocessing(CForm $form, array $preprocessing, $readonly, arra
 				);
 				break;
 
-			// ZBX-16642
 			case ZBX_PREPROC_CSV_TO_JSON:
 				$step_param_2_value = (array_key_exists('params', $step) && array_key_exists(2, $step['params']))
 					? $step['params'][2]
@@ -1445,7 +1509,7 @@ function getItemPreprocessing(CForm $form, array $preprocessing, $readonly, arra
 		$preprocessing_list->addItem(
 			(new CListItem([
 				(new CDiv([
-					(new CDiv())
+					(new CDiv(new CVar('preprocessing['.$i.'][sortorder]', $step['sortorder'])))
 						->addClass(ZBX_STYLE_DRAG_ICON)
 						->addClass(!$sortable ? ZBX_STYLE_DISABLED : null),
 					(new CDiv($preproc_types_select))
@@ -1678,7 +1742,7 @@ function getTriggerFormData(array $data) {
 		// Get templates.
 		$data['templates'] = makeTriggerTemplatesHtml($trigger['triggerid'],
 			getTriggerParentTemplates([$trigger], $flag), $flag,
-			CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES), $data['context']
+			CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES)
 		);
 
 		if ($data['show_inherited_tags']) {
