@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 /**
  * @var CView $this
+ * @var array $data
  */
 
 $widget = (new CWidget())->setTitle($data['title']);
@@ -84,3 +85,12 @@ $widget->addItem($form);
 require_once dirname(__FILE__).'/js/configuration.copy.elements.js.php';
 
 $widget->show();
+
+(new CScriptTag('
+	view.init('.json_encode([
+		'form_name' => $form->getName(),
+		'copy_targetids' => $data['copy_targetids']
+	]).');
+'))
+	->setOnDocumentReady()
+	->show();
