@@ -30,6 +30,7 @@ $triggersWidget = (new CWidget())
 	->setNavigation(getHostNavigation('triggers', $data['hostid'], $data['parent_discoveryid']));
 
 $url = (new CUrl('trigger_prototypes.php'))
+	->setArgument('parent_discoveryid', $data['parent_discoveryid'])
 	->setArgument('context', $data['context'])
 	->getUrl();
 
@@ -45,7 +46,8 @@ $triggersForm = (new CForm('post', $url))
 	->addVar('toggle_expression_constructor', '')
 	->addVar('toggle_recovery_expression_constructor', '')
 	->addVar('remove_expression', '')
-	->addVar('remove_recovery_expression', '');
+	->addVar('remove_recovery_expression', '')
+	->addVar('backurl', $data['backurl']);
 
 if ($data['triggerid'] !== null) {
 	$triggersForm->addVar('triggerid', $data['triggerid']);
@@ -86,7 +88,7 @@ $triggersFormList
 	)
 	->addRow(
 		_('Severity'),
-		new CSeverity(['name' => 'priority', 'value' => (int) $data['priority']])
+		new CSeverity('priority', (int) $data['priority'])
 	);
 
 // append expression to form list
@@ -110,9 +112,11 @@ if ($data['hostid']) {
 }
 $add_expression_button = (new CButton('insert', ($data['expression_constructor'] == IM_TREE) ? _('Edit') : _('Add')))
 	->addClass(ZBX_STYLE_BTN_GREY)
-	->onClick('return PopUp("popup.triggerexpr",jQuery.extend('.
-		json_encode($popup_options).
-			',{expression: jQuery(\'[name="'.$data['expression_field_name'].'"]\').val()}), null, this);'
+	->onClick(
+		'return PopUp("popup.triggerexpr", jQuery.extend('.json_encode($popup_options).', {
+				expression: jQuery(\'[name="'.$data['expression_field_name'].'"]\').val()
+			}), {dialogue_class: "modal-popup-generic"}
+		);'
 	)
 	->removeId();
 if ($data['limited']) {
@@ -262,8 +266,11 @@ if ($data['expression_constructor'] == IM_TREE) {
 	}
 
 	$testButton = (new CButton('test_expression', _('Test')))
-		->onClick('return PopUp("popup.testtriggerexpr",{expression: this.form.elements["expression"].value}, null,'.
-					'this);')
+		->onClick(
+			'return PopUp("popup.testtriggerexpr", {expression: this.form.elements["expression"].value}, {
+				dialogue_class: "modal-popup-generic"
+			});'
+		)
 		->addClass(ZBX_STYLE_BTN_LINK)
 		->removeId();
 	if (!$allowed_testing) {
@@ -305,15 +312,16 @@ $add_recovery_expression_button = (new CButton('insert',
 		($data['recovery_expression_constructor'] == IM_TREE) ? _('Edit') : _('Add'))
 	)
 	->addClass(ZBX_STYLE_BTN_GREY)
-	->onClick('return PopUp("popup.triggerexpr",jQuery.extend('.
-		json_encode([
+	->onClick(
+		'return PopUp("popup.triggerexpr", jQuery.extend('.json_encode([
 			'srctbl' => $data['recovery_expression_field_name'],
 			'srcfld1' => $data['recovery_expression_field_name'],
 			'dstfrm' => $triggersForm->getName(),
 			'dstfld1' => $data['recovery_expression_field_name'],
 			'parent_discoveryid' => $data['parent_discoveryid']
-		]).
-			',{expression: jQuery(\'[name="'.$data['recovery_expression_field_name'].'"]\').val()}), null, this);'
+		]).', {expression: jQuery(\'[name="'.$data['recovery_expression_field_name'].'"]\').val()}),
+			{dialogue_class: "modal-popup-generic"}
+		);'
 	);
 
 if ($data['limited']) {
@@ -458,8 +466,11 @@ if ($data['recovery_expression_constructor'] == IM_TREE) {
 	}
 
 	$testButton = (new CButton('test_expression', _('Test')))
-		->onClick('return PopUp("popup.testtriggerexpr",'.
-			'{expression: this.form.elements["recovery_expression"].value}, null, this);')
+		->onClick(
+			'return PopUp("popup.testtriggerexpr", {expression: this.form.elements["recovery_expression"].value}, {
+				dialogue_class: "modal-popup-generic"
+			});'
+		)
 		->addClass(ZBX_STYLE_BTN_LINK)
 		->removeId();
 	if (!$allowed_testing) {
@@ -552,7 +563,8 @@ $triggersTab->addTab('tags-tab', _('Tags'), new CPartial('configuration.tags.tab
 		'source' => 'trigger_prototype',
 		'tags' => $data['tags'],
 		'show_inherited_tags' => $data['show_inherited_tags'],
-		'readonly' => false
+		'readonly' => false,
+		'tabs_id' => 'tabs'
 	]), TAB_INDICATOR_TAGS
 );
 
@@ -593,7 +605,7 @@ foreach ($data['db_dependencies'] as $dependency) {
 	$row = new CRow([$description,
 		(new CCol(
 			(new CButton('remove', _('Remove')))
-				->onClick('javascript: removeDependency("'.$dependency['triggerid'].'");')
+				->onClick('view.removeDependency('.json_encode($dependency['triggerid']).')')
 				->addClass(ZBX_STYLE_BTN_LINK)
 				->removeId()
 		))->addClass(ZBX_STYLE_NOWRAP)
@@ -608,8 +620,8 @@ $dependenciesFormList->addRow(_('Dependencies'),
 		$dependenciesTable,
 		new CHorList([
 			(new CButton('add_dep_trigger', _('Add')))
-				->onClick('return PopUp("popup.generic",'.
-					json_encode([
+				->onClick(
+					'return PopUp("popup.generic", '.json_encode([
 						'srctbl' => 'triggers',
 						'srcfld1' => 'triggerid',
 						'reference' => 'deptrigger',
@@ -618,18 +630,18 @@ $dependenciesFormList->addRow(_('Dependencies'),
 						'normal_only' => '1',
 						'noempty' => '1',
 						'hostid' => $data['hostid']
-					]).', null, this);'
+					]).', {dialogue_class: "modal-popup-generic"});'
 				)
 				->addClass(ZBX_STYLE_BTN_LINK),
 			(new CButton('add_dep_trigger_prototype', _('Add prototype')))
-				->onClick('return PopUp("popup.generic",'.
-					json_encode([
+				->onClick(
+					'return PopUp("popup.generic", '.json_encode([
 						'srctbl' => 'trigger_prototypes',
 						'srcfld1' => 'triggerid',
 						'reference' => 'deptrigger',
 						'multiselect' => '1',
 						'parent_discoveryid' => $data['parent_discoveryid']
-					]).', null, this);'
+					]).', {dialogue_class: "modal-popup-generic"});'
 				)
 				->addClass(ZBX_STYLE_BTN_LINK)
 		])
@@ -639,10 +651,14 @@ $dependenciesFormList->addRow(_('Dependencies'),
 );
 $triggersTab->addTab('dependenciesTab', _('Dependencies'), $dependenciesFormList, TAB_INDICATOR_DEPENDENCY);
 
+$cancelButton = $data['backurl'] !== null
+	? new CButtonCancel(null, "redirect('".$data['backurl']."');")
+	: new CButtonCancel(url_params(['parent_discoveryid', 'context']));
+
 // append buttons to form
 if (!empty($data['triggerid'])) {
 	$deleteButton = new CButtonDelete(_('Delete trigger prototype?'),
-		url_params(['form', 'triggerid', 'parent_discoveryid', 'context']), 'context'
+		url_params(['form', 'triggerid', 'parent_discoveryid', 'context', 'backurl']), 'context'
 	);
 
 	if ($data['limited']) {
@@ -653,15 +669,14 @@ if (!empty($data['triggerid'])) {
 		new CSubmit('update', _('Update')), [
 			new CSubmit('clone', _('Clone')),
 			$deleteButton,
-			new CButtonCancel(url_params(['parent_discoveryid', 'context']))
+			$cancelButton
 		]
 	));
 }
 else {
 	$triggersTab->setFooter(makeFormFooter(
 		new CSubmit('add', _('Add')),
-		[new CButtonCancel(url_params(['parent_discoveryid', 'context']))]
-	));
+		[$cancelButton]));
 }
 
 // append tabs to form
@@ -670,3 +685,11 @@ $triggersForm->addItem($triggersTab);
 $triggersWidget->addItem($triggersForm);
 
 $triggersWidget->show();
+
+(new CScriptTag('
+	view.init('.json_encode([
+		'form_name' => $triggersForm->getName()
+	]).');
+'))
+	->setOnDocumentReady()
+	->show();
