@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -134,7 +134,7 @@ $icon_mapping_link = (new CLink(_('show icon mappings'), (new CUrl('zabbix.php')
 		->setArgument('action', 'iconmap.list')
 		->getUrl()
 	))
-	->setAttribute('target', '_blank');
+	->setTarget('_blank');
 $map_tab->addRow(new CLabel(_('Automatic icon mapping'), $icon_mapping->getFocusableElementId()),
 	[$icon_mapping, SPACE, $icon_mapping_link]
 );
@@ -265,7 +265,7 @@ $map_tab->addRow(new CLabel(_('Problem display'), 'label-show-unack'),
 );
 
 $map_tab->addRow(_('Minimum severity'),
-	new CSeverity(['name' => 'severity_min', 'value' => (int) $data['sysmap']['severity_min']])
+	new CSeverity('severity_min', (int) $data['sysmap']['severity_min'])
 );
 
 $map_tab->addRow(_('Show suppressed problems'),
@@ -323,14 +323,14 @@ $user_group_shares_table = (new CTable())
 	->setAttribute('style', 'width: 100%;');
 
 $add_user_group_btn = ([(new CButton(null, _('Add')))
-	->onClick('return PopUp("popup.generic",'.
-		json_encode([
+	->onClick(
+		'return PopUp("popup.generic", '.json_encode([
 			'srctbl' => 'usrgrp',
 			'srcfld1' => 'usrgrpid',
 			'srcfld2' => 'name',
 			'dstfrm' => $form->getName(),
 			'multiselect' => '1'
-		]).', null, this);'
+		]).', {dialogue_class: "modal-popup-generic"});'
 	)
 	->addClass(ZBX_STYLE_BTN_LINK)]);
 
@@ -351,7 +351,7 @@ foreach ($data['sysmap']['userGroups'] as $user_group) {
 	];
 }
 
-$js_insert = 'window.addPopupValues('.zbx_jsvalue(['object' => 'usrgrpid', 'values' => $user_groups]).');';
+$js_insert = 'window.addPopupValues('.json_encode(['object' => 'usrgrpid', 'values' => $user_groups]).');';
 
 // User sharing table.
 $user_shares_table = (new CTable())
@@ -360,14 +360,14 @@ $user_shares_table = (new CTable())
 	->setAttribute('style', 'width: 100%;');
 
 $add_user_btn = ([(new CButton(null, _('Add')))
-	->onClick('return PopUp("popup.generic",'.
-		json_encode([
+	->onClick(
+		'return PopUp("popup.generic", '.json_encode([
 			'srctbl' => 'users',
 			'srcfld1' => 'userid',
 			'srcfld2' => 'fullname',
 			'dstfrm' => $form->getName(),
 			'multiselect' => '1'
-		]).', null, this);'
+		]).', {dialogue_class: "modal-popup-generic"});'
 	)
 	->addClass(ZBX_STYLE_BTN_LINK)]);
 
