@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -18,11 +18,14 @@
 **/
 
 
+/**
+ * @event resize - Event fired on textarea size change.
+ */
 (function($) {
 	'use strict';
 
 	function update(e) {
-		var $textarea = $(this);
+		const $textarea = $(this);
 
 		if (e.which === 13) {
 			// Simulate input behavior by submitting form on enter key.
@@ -40,15 +43,31 @@
 			return false;
 		}
 
+		/**
+		 * Simulate input behaviour by replacing newlines with space character.
+		 * NB! WebKit based browsers add a newline character to textarea when translating content to the next line.
+		 */
 		var old_value = $textarea.val(),
-			new_value = old_value.replace(/\r?\n/gi, ''),
+			new_value = old_value
+				.replace(/\r?\n+$/g, '')
+				.replace(/\r?\n/g, ' '),
 			scroll_pos = $(window).scrollTop();
 
-		if (old_value.length !== new_value.length) {
+		if (old_value !== new_value) {
+			var pos = $textarea[0].selectionStart;
+
 			$textarea.val(new_value);
+			$textarea[0].setSelectionRange(pos, pos);
 		}
 
-		$textarea.height(0).innerHeight($textarea[0].scrollHeight);
+		// Resize textarea.
+		$textarea
+			.height(0)
+			.innerHeight($textarea[0].scrollHeight);
+
+		// Fire event.
+		$textarea.trigger('resize');
+
 		$(window).scrollTop(scroll_pos);
 	}
 
@@ -64,6 +83,15 @@
 					.on('input keydown paste', update)
 					.trigger('input');
 			});
+		},
+		clean: function() {
+			return this.each(function() {
+				var $textarea = $(this);
+
+				$textarea
+					.val('')
+					.css('height', '');
+			});
 		}
 	};
 
