@@ -26,9 +26,7 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
 $page['title'] = _('Configuration of templates');
 $page['file'] = 'templates.php';
-$page['scripts'] = ['multiselect.js', 'textareaflexible.js', 'inputsecret.js', 'macrovalue.js',
-	'class.tab-indicators.js', 'class.tagfilteritem.js'
-];
+$page['scripts'] = ['class.tagfilteritem.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -194,15 +192,15 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	try {
 		DBstart();
 
-		$templateId = getRequest('templateid', 0);
+		$input_templateid = getRequest('templateid', 0);
 		$cloneTemplateId = 0;
 
 		if (getRequest('form') === 'full_clone') {
-			$cloneTemplateId = $templateId;
-			$templateId = 0;
+			$cloneTemplateId = $input_templateid;
+			$input_templateid = 0;
 		}
 
-		if ($templateId == 0) {
+		if ($input_templateid == 0) {
 			$messageSuccess = _('Template added');
 			$messageFailed = _('Cannot add template');
 		}
@@ -239,41 +237,41 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			$templates[] = ['templateid' => $templateid];
 		}
 
-		$templatesClear = getRequest('clear_templates', []);
-		$templatesClear = zbx_toObject($templatesClear, 'templateid');
-		$templateName = getRequest('template_name', '');
+		$template_name = getRequest('template_name', '');
 
 		// create / update template
 		$template = [
-			'host' => $templateName,
-			'name' => getRequest('visiblename', ''),
+			'host' => $template_name,
+			'name' => (getRequest('visiblename', '') === '') ? $template_name : getRequest('visiblename'),
+			'description' => getRequest('description', ''),
 			'groups' => zbx_toObject($groups, 'groupid'),
 			'templates' => $templates,
-			'macros' => $macros,
 			'tags' => $tags,
-			'description' => getRequest('description', '')
+			'macros' => $macros
 		];
 
-		if ($templateId == 0) {
+		if ($input_templateid == 0) {
 			$result = API::Template()->create($template);
 
 			if ($result) {
-				$templateId = reset($result['templateids']);
+				$input_templateid = reset($result['templateids']);
 			}
 			else {
 				throw new Exception();
 			}
 		}
 		else {
-			$template['templateid'] = $templateId;
-			$template['templates_clear'] = $templatesClear;
+			$templates_clear = array_diff(
+				getRequest('clear_templates', []),
+				getRequest('add_templates', [])
+			);
+
+			$template['templateid'] = $input_templateid;
+			$template['templates_clear'] = zbx_toObject($templates_clear, 'templateid');
 
 			$result = API::Template()->update($template);
 
-			if ($result) {
-				add_audit_ext(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_TEMPLATE, $templateId, $templateName, 'hosts', null, null);
-			}
-			else {
+			if (!$result) {
 				throw new Exception();
 			}
 		}
@@ -292,7 +290,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		else if (hasRequest('update')) {
 			$del_valuemapids = API::ValueMap()->get([
 				'output' => [],
-				'hostids' => $templateId,
+				'hostids' => $input_templateid,
 				'preservekeys' => true
 			]);
 		}
@@ -303,7 +301,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 				unset($del_valuemapids[$valuemap['valuemapid']]);
 			}
 			else {
-				$ins_valuemaps[] = $valuemap + ['hostid' => $templateId];
+				$ins_valuemaps[] = $valuemap + ['hostid' => $input_templateid];
 			}
 		}
 
@@ -326,11 +324,11 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			 * First copy web scenarios with web items, so that later regular items can use web item as their master
 			 * item.
 			 */
-			if (!copyHttpTests($cloneTemplateId, $templateId)) {
+			if (!copyHttpTests($cloneTemplateId, $input_templateid)) {
 				throw new Exception();
 			}
 
-			if (!copyItems($cloneTemplateId, $templateId)) {
+			if (!copyItems($cloneTemplateId, $input_templateid, true)) {
 				throw new Exception();
 			}
 
@@ -342,10 +340,8 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			]);
 
 			if ($dbTriggers) {
-				$result &= copyTriggersToHosts(zbx_objectValues($dbTriggers, 'triggerid'),
-						$templateId, $cloneTemplateId);
-
-				if (!$result) {
+				if (!copyTriggersToHosts(zbx_objectValues($dbTriggers, 'triggerid'), $input_templateid,
+						$cloneTemplateId)) {
 					throw new Exception();
 				}
 			}
@@ -358,7 +354,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			]);
 
 			foreach ($dbGraphs as $dbGraph) {
-				copyGraphToHost($dbGraph['graphid'], $templateId);
+				copyGraphToHost($dbGraph['graphid'], $input_templateid);
 			}
 
 			// copy discovery rules
@@ -369,10 +365,12 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			]);
 
 			if ($dbDiscoveryRules) {
-				$result &= API::DiscoveryRule()->copy([
+				if (!API::DiscoveryRule()->copy([
 					'discoveryids' => zbx_objectValues($dbDiscoveryRules, 'itemid'),
-					'hostids' => [$templateId]
-				]);
+					'hostids' => [$input_templateid]
+				])) {
+					$result = false;
+				}
 
 				if (!$result) {
 					throw new Exception();
@@ -388,7 +386,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			]);
 
 			if ($db_template_dashboards) {
-				$db_template_dashboards = CDashboardHelper::prepareForClone($db_template_dashboards, $templateId);
+				$db_template_dashboards = CDashboardHelper::prepareForClone($db_template_dashboards, $input_templateid);
 
 				if (!API::TemplateDashboard()->create($db_template_dashboards)) {
 					throw new Exception();
@@ -410,22 +408,56 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	}
 }
 elseif (hasRequest('templateid') && hasRequest('delete')) {
-	DBstart();
+	try {
+		DBstart();
+
+		$hosts = API::Host()->get([
+			'output' => [],
+			'templateids' => getRequest('templateid'),
+			'preservekeys' => true
+		]);
+
+		if ($hosts) {
+			$result = API::Host()->massRemove([
+				'hostids' => array_keys($hosts),
+				'templateids' => getRequest('templateid')
+			]);
+
+			if (!$result) {
+				throw new Exception();
+			}
+		}
+
+		$templates = API::Template()->get([
+			'output' => [],
+			'parentTemplateids' => getRequest('templateid'),
+			'preservekeys' => true
+		]);
+
+		if ($templates) {
+			$result = API::Template()->massRemove([
+				'templateids' => array_keys($templates),
+				'templateids_link' => getRequest('templateid')
+			]);
+
+			if (!$result) {
+				throw new Exception();
+			}
+		}
 
-	$result = API::Template()->massUpdate([
-		'templates' => zbx_toObject($_REQUEST['templateid'], 'templateid'),
-		'hosts' => []
-	]);
-	if ($result) {
 		$result = API::Template()->delete([getRequest('templateid')]);
-	}
 
-	$result = DBend($result);
+		$result = DBend($result);
+	}
+	catch (Exception $e) {
+		DBend(false);
+	}
 
 	if ($result) {
 		unset($_REQUEST['form'], $_REQUEST['templateid']);
 		uncheckTableRows();
 	}
+
 	unset($_REQUEST['delete']);
 	show_messages($result, _('Template deleted'), _('Cannot delete template'));
 }
@@ -444,36 +476,69 @@ elseif (hasRequest('templateid') && hasRequest('delete_and_clear')) {
 	show_messages($result, _('Template deleted'), _('Cannot delete template'));
 }
 elseif (hasRequest('templates') && hasRequest('action') && str_in_array(getRequest('action'), ['template.massdelete', 'template.massdeleteclear'])) {
-	$templates = getRequest('templates');
+	try {
+		DBstart();
 
-	DBstart();
+		$templateids = getRequest('templates');
 
-	$result = true;
+		if (getRequest('action') === 'template.massdelete') {
+			$hosts = API::Host()->get([
+				'output' => [],
+				'templateids' => $templateids,
+				'preservekeys' => true
+			]);
 
-	if (getRequest('action') === 'template.massdelete') {
-		$result = API::Template()->massUpdate([
-			'templates' => zbx_toObject($templates, 'templateid'),
-			'hosts' => []
-		]);
+			if ($hosts) {
+				$result = API::Host()->massRemove([
+					'hostids' => array_keys($hosts),
+					'templateids' => $templateids
+				]);
+
+				if (!$result) {
+					throw new Exception();
+				}
+			}
+
+			$templates = API::Template()->get([
+				'output' => [],
+				'parentTemplateids' => $templateids,
+				'preservekeys' => true
+			]);
+
+			if ($templates) {
+				$result = API::Template()->massRemove([
+					'templateids' => array_keys($templates),
+					'templateids_link' => $templateids
+				]);
+
+				if (!$result) {
+					throw new Exception();
+				}
+			}
+		}
+
+		$result = API::Template()->delete($templateids);
+
+		$result = DBend($result);
 	}
-
-	if ($result) {
-		$result = API::Template()->delete($templates);
+	catch (Exception $e) {
+		DBend(false);
 	}
 
-	$result = DBend($result);
-
 	if ($result) {
 		uncheckTableRows();
 	}
 	else {
-		$templateids = API::Template()->get([
+		$templates = API::Template()->get([
 			'output' => [],
-			'templateids' => $templates,
-			'editable' => true
+			'templateids' => $templateids,
+			'editable' => true,
+			'preservekeys' => true
 		]);
-		uncheckTableRows(null, zbx_objectValues($templateids, 'templateid'));
+
+		uncheckTableRows(null, array_keys($templates));
 	}
+
 	show_messages($result, _('Template deleted'), _('Cannot delete template'));
 }
 
@@ -648,6 +713,29 @@ if (hasRequest('form')) {
 		'add_templates' => array_map('strval', array_keys($data['add_templates']))
 	];
 
+	$data['template_name'] = getRequest('template_name', '');
+	$data['visible_name'] = getRequest('visiblename', '');
+
+	$templateids = getRequest('templates', []);
+	$clear_templates = getRequest('clear_templates', []);
+
+	if ($data['templateid'] != 0 && !hasRequest('form_refresh')) {
+		$data['template_name'] = $data['dbTemplate']['host'];
+		$data['visible_name'] = $data['dbTemplate']['name'];
+
+		// Display empty visible name if equal to host name.
+		if ($data['visible_name'] === $data['template_name']) {
+			$data['visible_name'] = '';
+		}
+
+		$templateids = $data['original_templates'];
+	}
+
+	$clear_templates = array_intersect($clear_templates, array_keys($data['original_templates']));
+	$clear_templates = array_diff($clear_templates, array_keys($templateids));
+
+	$data['clear_templates'] = $clear_templates;
+
 	$view = new CView('configuration.template.edit', $data);
 }
 else {
